@@ -25,11 +25,12 @@ from platform import uname
 class MDSimulation():
     def __init__(self, parmed_structure, coordinates=None,
                  velocities=None, box_vectors=None,
-                 cwd=None, centering=True,
+                 cwd=None, centering=False,
                  temperature=300 * unit.kelvin, pressure=1 * unit.atmosphere, membrane=False,
                  integrator_type='Langevin', platform_name=None, nonbonded_method='PME', nonbonded_cutoff=1 * unit.nanometer,
                  precision=None,
                  atoms_to_freeze=None, atoms_to_restrain=None,
+                 use_COM_restraint=False,
                  restraint_weight=None, constraints='HBonds',
                  state_out='sim.log',
                  implicit_solvent_model=None, hmr=False,
@@ -182,6 +183,7 @@ class MDSimulation():
         self.cwd = cwd
 
         self.atoms_to_freeze = atoms_to_freeze
+        self.use_COM_restraint =use_COM_restraint
         self.platform_name = platform_name
         self.precision = precision
         self.temperature = temperature
@@ -376,7 +378,7 @@ class MDSimulation():
 
         print('coordinates being used for COM restraint: {}'.format(com))
 
-        restraint = mm.CustomCentroidBondForce(1, 'K*sqrt((x1-x0)^2 + (y1-y0)^2 + (z1-z0)^2)')
+        restraint = mm.CustomCentroidBondForce(1, 'K*(sqrt((x1-x0)^2 + (y1-y0)^2 + (z1-z0)^2))^2')
         restraint.addGlobalParameter('K', COM_restraint_weight)
         restraint.addGlobalParameter('x0', com[0])
         restraint.addGlobalParameter('y0', com[1])
@@ -403,7 +405,6 @@ class MDSimulation():
         new_coords = self.coordinates + delta
         self.positions = new_coords * unit.angstrom
 
-
     def _build_sim(self):
         """Use params specified on init to build the simulation context, specify the platform and bind an integrator
 
@@ -414,7 +415,7 @@ class MDSimulation():
             print('created directory {} for storing sim files'.format(self.cwd))
         #make sure the cwd exists in case we're unpickling
         elif not os.path.exists(self.cwd):
-            self.cwd = os.mkdir(self.cwd)
+            os.mkdir(self.cwd)
             print('storing sim files in directory {}'.format(self.cwd))
         else:
             print('storing sim files in directory {}'.format(self.cwd))
@@ -505,7 +506,12 @@ class MDSimulation():
         if self.atoms_to_restrain is not None:
             self.set_positional_restraints(self.atoms_to_restrain, self.restraint_weight)
 
+        if self.restrain_COM:
+            atoms = select_atoms(self.parmed_structure, keyword_selection='protein ligand')
+            self.restrain_COM(atoms, 20)
 
+    def _build_custom_forces(self):
+        pass
 
     def _write_coordinates_to_h5(self, fname='current_frame.h5'):
         """Writes minimized coordinates to an H5 file
@@ -885,6 +891,7 @@ class MetadynamicSimulation(MDSimulation):
 
 
     """
+
     def __init__(self, CV_list=[],
                  funnel=False, radius_scale_factor=2,
                  dome=False, funnel_angle=.5,
@@ -903,17 +910,26 @@ class MetadynamicSimulation(MDSimulation):
         self.bias_factor = bias_factor
         self.initial_bias_height = initial_bias_height
         self.bias_frequency = int(bias_frequency / self.step_length)
+        self.bias_directory = bias_directory
         self.bias_save_frequency = int(bias_save_frequency / self.step_length)
         self.fes_interval = int(fes_interval / self.step_length)
+        self.dome = dome
+        self.funnel = funnel
+        self.radius_scale_factor = radius_scale_factor
+        self.funnel_angle = funnel_angle
 
-        if bias_directory is not None:
-            self.bias_directory = bias_directory
-        else:
+        self._build_custom_forces()
+
+
+    def _build_custom_forces(self):
+
+        #TODO it would probably be better to have a separate utilities call
+        if self.bias_directory is None:
             self.bias_directory = self.cwd
 
 
-        # build funnel or dome restraints in init
-        if dome:
+        # build funnel or dome restraints
+        if self.dome:
             if self.membrane:
                 raise ValueError(
                     'Spherical restraints should not be used for channel protein systems with deep binding pockets.')
@@ -927,7 +943,7 @@ class MetadynamicSimulation(MDSimulation):
             if len(lig_atom_index) == 0 or len(prot_atom_index) == 0:
                 raise ValueError('could not select ligand or protein for dome placement')
 
-            radius, center = find_radius_and_center(lig_positions, radius_scale_factor)
+            radius, center = find_radius_and_center(lig_positions, self.radius_scale_factor)
 
             print('Adding a spherical potential to keep ligand near binding site ',
                   'using the following params',
@@ -945,12 +961,16 @@ class MetadynamicSimulation(MDSimulation):
 
             self.custom_forces['dome'] = self.system.addForce(spherical_boundary_force)
 
-            # use the inherited restrain COM to keep the protein from floating away
-            self.restrain_COM(atom_selection=prot_atom_index, COM_restraint_weight=10)
+            print('applying dome to {} num of atoms'.format(len(lig_atom_index)))
+            for i in lig_atom_index:
+                spherical_boundary_force.addParticle(i, [])
 
-        if funnel:
-            if not self.membrane:
-                raise ValueError('funnel for non-membrane systems not yet supported.')
+            # use the inherited restrain COM to keep the protein from floating away
+            self.restrain_COM(atom_selection=prot_atom_index, COM_restraint_weight=30)
+
+
+        if self.funnel:
+            print ('Warning! Funnel should only be used for correctly aligned structures.')
 
             # Now get the setup stage for placing funnel (otherwise coords may be wrapped)
             lig_atom_index = select_atoms(self.parmed_structure, keyword_selection='ligand')
@@ -973,14 +993,15 @@ class MetadynamicSimulation(MDSimulation):
                 z_cone_apex = 0
                 orient_factor = 1
 
-            r_cyl, center = find_radius_and_center(lig_positions, radius_scale_factor)
+            r_cyl, center = find_radius_and_center(lig_positions, self.radius_scale_factor)
 
             print('Adding a funnel potential to keep ligand near channel',
                   'mouth using the following params',
-                  'r_cyl: {} zcc: {} alpha: {}, ligand COG {} '.format(r_cyl, z_cone_apex, funnel_angle, center))
-            print('Note! This funnel assumes that the channel protein extracellular domain is aligned ',
+                  'r_cyl: {} zcc: {} alpha: {}, ligand COG {} '.format(r_cyl, z_cone_apex, self.funnel_angle, center))
+            print('Note! This funnel assumes that the ligand exit along the path is generally aligned ',
                   'with the z-axis')
 
+
             # the funnel shape depends on the radius and the x,y as function of z, needs to use periodicdistance()
             funnel_force = mm.CustomExternalForce(
                 'K*max(0, (r-radius))^2; r=sqrt(periodicdistance(x,y,z*0,x0,y0,zcc*0)^2); '
@@ -990,7 +1011,7 @@ class MetadynamicSimulation(MDSimulation):
             funnel_force.addGlobalParameter('r_cyl', r_cyl)
             funnel_force.addGlobalParameter('zcc', z_cone_apex)
             funnel_force.addGlobalParameter('orient_factor', orient_factor)
-            funnel_force.addGlobalParameter('alpha', funnel_angle)
+            funnel_force.addGlobalParameter('alpha', self.funnel_angle)
             funnel_force.addGlobalParameter('x0', center[0])
             funnel_force.addGlobalParameter('y0', center[1])
             self.custom_forces['funnel'] = self.system.addForce(funnel_force)
@@ -1001,15 +1022,15 @@ class MetadynamicSimulation(MDSimulation):
         self.bin_lengths = []
         # cv list format will be:
         #       cv[0] = cv name
-        #       cv[1] = atom_selection1
-        #       cv[2] = atom_selection2
+        #       cv[1] = atom_selection1 (should be ligand if using both)
+        #       cv[2] = atom_selection2 (should be protein if possible)
         #       cv[3] = cv min value
         #       cv[4] = cv max value
         #       cv[5] = cv bin width
 
-        print('Building collective variables with the following specifications: {}'.format(CV_list))
+        print('Building collective variables with the following specifications: {}'.format(self.CV_list))
 
-        for cv in CV_list:
+        for cv in self.CV_list:
             if cv[0] == 'RMSD':
 
                 if cv[1].startswith('resid'):
@@ -1135,6 +1156,36 @@ class MetadynamicSimulation(MDSimulation):
         self.simulation.context.setPositions(self.positions)
 
 
+    def pickle_sim(self, pickle_name='mdsim.pickle'):
+        """ Remove the Swigpy objects that don't pickle and write the current simulation pickle to cwd
+        In order to use the pickle you will need to run _build_sim
+
+        Examples
+        -------
+        >>> mdsim.pickle_sim()
+        >>> with open(os.path.join(mdsim.cwd,'mdsim.pickle'), 'rb') as handle:
+        >>>     new_sim = pickle.load(handle)
+        >>>
+        >>> new_sim._build_sim()
+        >>> new_sim._build_custom_forces()
+        >>> new_sim.run()
+        """
+
+        if (self.progress_out == stdout) or (self.state_out == stdout):
+            raise ValueError('cant pickle system if any logs point to stdout!')
+
+        print('removing sim and system objects in order to pickle')
+        del self.simulation
+        del self.system
+        del self.platform
+        del self.metaD
+
+        fname = os.path.join(self.cwd, pickle_name)
+        with open(fname, 'wb') as handle:
+            pickle.dump(obj=self, file=handle)
+        print('pickled sim saved as {}'.format(fname))
+        print('to use it once unpickled you will need to run self._build_sim() and self._build_custom_forces()')
+
     def _add_FESReporter(self):
         """
         adds the FES reporter to the sim
@@ -1228,16 +1279,19 @@ class SimulatedTemperingSimulation(MDSimulation):
         return self.parmed_structure
 
 
+
+
+
 if __name__ == '__main__':
     import _pickle as pickle
 
     import sys
 
-    if len(sys.argv[1:]) != 2:
-        print("Usage: python rs_simtools.py pickled_sim end_parmed")
+    if len(sys.argv[1:]) < 2 or len(sys.argv[1:]) > 3:
+        print("Usage: python rs_simtools.py pickled_sim end_parmed [time]")
         exit(1)
     else:
-        pickled_sim, new_parmed_file = sys.argv[1:]
+        pickled_sim, new_parmed_file = sys.argv[1:3]
 
         try:
             with open(pickled_sim, 'rb') as handle:
@@ -1248,7 +1302,14 @@ if __name__ == '__main__':
         mdsim.cwd = os.getcwd()
         mdsim.progress_out = sys.stdout
         mdsim._build_sim()
+        mdsim._build_custom_forces()
 
+        if len(sys.argv[1:]) == 2:
+            new_parmed = mdsim.run()
+        else:
+            time = sys.argv[3]
+            new_parmed = mdsim.run(time=time * unit.nanosecond)
+        
         new_parmed = mdsim.run()
 
         with open(new_parmed_file,'wb') as handle:
